"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Value = void 0;
var debounce = require("lodash.debounce");
var memory_storage_1 = require("./memory-storage");
var Value = /** @class */ (function () {
    function Value(key, opt) {
        if (opt === void 0) { opt = {}; }
        var defaultStorage = Value._isLocalStorageAvailable() ? localStorage : memory_storage_1.default;
        this.storage = typeof opt.storage !== 'undefined' ? opt.storage : defaultStorage;
        this.defaults = typeof opt.default !== 'undefined' ? opt.default : null;
        this.expires = typeof opt.expires === 'number' ? opt.expires : null;
        this.debouncedFlush = debounce(this.flush, opt.debounceTime ? opt.debounceTime : 200);
        this.namespace = typeof opt.namespace === 'string' ? opt.namespace + "." : '';
        this.key = "" + this.namespace + key;
        this.storageId = Value._storages.indexOf(this.storage);
        // 新たな storage を追加された場合は管理対象に追加
        if (this.storageId < 0) {
            this.storageId = Value._storages.length;
            Value._storages[this.storageId] = this.storage;
            Value._values[this.storageId] = {};
            Value._expires[this.storageId] = {};
            Value.gc();
        }
        // メモリ上に値がなければ storage から取り出す
        if (typeof Value._values[this.storageId][this.key] === 'undefined') {
            try {
                Value._values[this.storageId][this.key] = JSON.parse(this.storage.getItem(this.key));
                Value._expires[this.storageId][this.key] = JSON.parse(this.storage.getItem(Value._expiresKeyGen(this.key)));
            }
            catch (error) {
                (console.error || console.log)('invalid value on storage-value', this.key, error);
            }
        }
    }
    /**
     * 全ての Value インスタンスを storage に保存
     */
    Value.flush = function () {
        Value._storages.forEach(function (storage, storageId) {
            Object.keys(Value._values[storageId]).forEach(function (key) {
                var value = Value._values[storageId][key];
                var expire = Value._expires[storageId][key];
                var expireKey = Value._expiresKeyGen(key);
                if (typeof value === 'undefined') {
                    return;
                }
                storage.setItem(key, JSON.stringify(value));
                expire ? storage.setItem(expireKey, JSON.stringify(expire)) : storage.removeItem(expireKey);
            });
        });
    };
    /**
     * 全ての Value インスタンスを初期化
     */
    Value.clear = function () {
        Value._storages.forEach(function (storage, storageId) {
            Object.keys(Value._values[storageId]).forEach(function (key) {
                storage.removeItem(key);
                storage.removeItem(Value._expiresKeyGen(key));
            });
        });
        Value._values = [];
        Value._expires = [];
        Value._storages = [];
    };
    /**
     * storage に保存されている期限切れデータを全て削除します。
     * どんな storage が使われているかは１つでもインスタンス化されてないとわからないので、
     * gc する storage は Value._storages にあるものだけを対象にします。
     */
    Value.gc = function () {
        var iterate = function (storage, callback) {
            if (typeof storage.key == 'function') {
                var keys = [];
                for (var i = 0; i < storage.length; ++i) {
                    var key = storage.key(i);
                    if (key) {
                        keys.push(key);
                    }
                }
                keys.forEach(function (key) { return callback(key); });
            }
            else {
                for (var key in storage) {
                    callback(key);
                }
            }
        };
        Value._storages.forEach(function (storage, storageId) {
            iterate(storage, function (key) {
                // expire 時刻を保存してあるキー名を調べる。
                var expiresKey = Value._expiresKeyGen(key);
                var expires;
                try {
                    expires = JSON.parse(storage.getItem(expiresKey));
                }
                catch (error) {
                    (console.error || console.log)('invalid expires on storage-value', expiresKey, error);
                    return;
                }
                // expires が数値でないときは何もしない
                if (typeof expires !== 'number') {
                    return;
                }
                // 期限が切れていない場合は何もしない
                if (expires > Date.now()) {
                    return;
                }
                // 期限切れの場合は、メモリと storage から削除
                delete Value._values[storageId][key];
                delete Value._expires[storageId][key];
                storage.removeItem(key);
                storage.removeItem(expiresKey);
            });
        });
    };
    /**
     * 有効期限用のキーを生成
     * @private
     */
    Value._expiresKeyGen = function (key) {
        return "EXPIRES." + key;
    };
    /**
     * localStorage が利用可能かどうか
     * @private
     */
    Value._isLocalStorageAvailable = function () {
        try {
            return (typeof localStorage !== 'undefined');
        }
        catch (e) { }
        return false;
    };
    /**
     * namespace 指定をした Value を返す
     */
    Value.namespace = function (name) {
        return function (key, opt) {
            if (opt === void 0) { opt = {}; }
            opt.namespace = name;
            return new Value(key, opt);
        };
    };
    /**
     * Value インスタンスの内容を storage に保存
     */
    Value.prototype.flush = function () {
        if (typeof Value._values[this.storageId] === 'undefined' || (typeof Value._values[this.storageId][this.key] === 'undefined')) {
            return;
        }
        this.storage.setItem(this.key, JSON.stringify(Value._values[this.storageId][this.key]));
        var expires = Value._expires[this.storageId][this.key];
        typeof expires === 'number' && this.storage.setItem(Value._expiresKeyGen(this.key), JSON.stringify(expires));
    };
    /**
     * Value インスタンスの内容を初期化
     * storage の内容も削除
     */
    Value.prototype.clear = function () {
        delete Value._values[this.storageId][this.key];
        delete Value._expires[this.storageId][this.key];
        this.storage.removeItem(this.key);
        this.storage.removeItem(Value._expiresKeyGen(this.key));
    };
    Object.defineProperty(Value.prototype, "useDefault", {
        get: function () {
            if (this.isExpired) {
                this.clear();
            }
            var value = Value._values[this.storageId][this.key];
            return (value === null || typeof value === 'undefined');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "value", {
        get: function () {
            if (this.isExpired) {
                this.clear();
            }
            return this.useDefault ? this.defaults : Value._values[this.storageId][this.key];
        },
        set: function (value) {
            Value._values[this.storageId][this.key] = value;
            if (this.expires !== null) {
                Value._expires[this.storageId][this.key] = Date.now() + this.expires;
            }
            this.debouncedFlush();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isExpired", {
        /**
         * 有効期限が切れているかどうか
         */
        get: function () {
            var expires = Value._expires[this.storageId][this.key];
            // そもそも有効期限が設定されていない場合は false
            if (expires === null) {
                return false;
            }
            if (Date.now() < expires) {
                return false;
            }
            return true;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * new Value によって作られた値の配列
     * @private
     */
    Value._values = [];
    /**
     * 有効期限を管理するオブジェクト
     * @private
     */
    Value._expires = [];
    /**
     * storage 指定によって指定された storage の配列
     * @private
     */
    Value._storages = [];
    return Value;
}());
exports.Value = Value;
exports.default = Value;
//# sourceMappingURL=value.js.map